"use strict";
var Controller = function(
 aVert,
 aIdx,
 arraySprites,
 glDemo
)
{	
if (aVert == 0){
 this.viewGetWebGL(null);
 return;
}
if (typeof Float32Array === 'undefined'){
 this.viewGetWebGL(null);
 return;
}
// Properties
// for sub data:
this.aI = aIdx;
this.aV = aVert;
 
this.responsive = null;
this.aSprites = arraySprites; 
this.glDemo = glDemo;
this.uMatrixTransform = null;
this.nImagesToLoad = Number(0);
this.nImagesLoaded = Number(0);

// Different
// 3D context names:
this.a3D = [
'webgl', 
'experimental-webgl', 
'webkit-3d',
'moz-webgl'
];

this.eDebug = document.getElementById("eDebug");
this.cv = document.getElementById('cv'); 
this.cv.controller = this;
 
if (glDemo.ctxAttrib != null){
 this.gl = this.getGLContextAndAttribs(glDemo.ctxAttrib);
}

else {
 this.gl = this.getGLContext();
}

if (this.gl == null){ 	
 this.viewGetWebGL("Error initializing WebGL.");
 return null;
}

this.program = this.getProgram();

if (this.program == null){
 this.viewGetWebGL("Error initializing WebGL.");
 return null;
}  

this.getProgramVariables();

this.getBuffers(
 aVert,
 aIdx
);

this.getImages();
return this;
};

Controller.prototype = {		
getProgram: function(){ 
var gl = this.gl;
var shaderF = this.getShader
(
"shader-f", 
gl.FRAGMENT_SHADER
);
var shaderV = this.getShader
(
"shader-v", 
gl.VERTEX_SHADER
);
var p = gl.createProgram();
gl.attachShader
(
p, 
shaderF
);

gl.attachShader
(
p, 
shaderV
);

gl.linkProgram
(
p
);

gl.useProgram
(
p
);

return p;
}, 

getBuffers: function(
aV,
aI
)
{	
var gl = this.gl; 
var bufferVT = gl.createBuffer();
gl.bindBuffer
(
 gl.ARRAY_BUFFER, 
 bufferVT
);
gl.bufferData
(
 gl.ARRAY_BUFFER, 
 aV, 
 gl.STATIC_DRAW
); 

gl.vertexAttribPointer
(
 this.aPosition, 
 3, 
 gl.FLOAT, 
 gl.FALSE, 
 20, 
 0
); 

var bufferIndices = gl.createBuffer(); 
gl.bindBuffer
(
 gl.ELEMENT_ARRAY_BUFFER,
 bufferIndices
);
gl.bufferData
(
 gl.ELEMENT_ARRAY_BUFFER, 
 aI, 
 gl.STATIC_DRAW
); 
},
  
/**
 * Prepare variables
 * from the WebGLProgram.
 */
getProgramVariables: function()
{
 
var gl = this.gl;

var program = this.program;

this.uMatrixPerspective = gl.getUniformLocation
(
 program,
 "um4_pmatrix"
);  

var aMP = new Float32Array(
[
 2.4,0,0,0,
 0,2.4,0,0, 
 0,0,-1,-1, 
 0,0,-0.2,0
]
);  

gl.uniformMatrix4fv
(
 this.uMatrixPerspective, 
 gl.FALSE, 
 aMP
);

this.uMatrixTransform = gl.getUniformLocation
(
 program, 
 "um4_matrix"
);

this.aPosition = gl.getAttribLocation
(
 program, 
 "a_position"
);

gl.enableVertexAttribArray
( 
 this.aPosition
); 

gl.viewport
(
 0, 
 0, 
 512,
 512
);

},

/**
 * Prepare textures
 * from image paths
 * within array
 * of Sprite.
 */
getImages: function() 
{

var aSprites = this.aSprites;   

var gl = this.gl;  

for (var i = 0; i < aSprites.length; i++){
 var sprite = aSprites[i];
 this.nImagesToLoad += sprite.getTextureVariables(
  this
);        
}  

},

/**
 * Validate and finalize
 * WebGLProgram.
 * @param {Controller} controller 
 */
setProgramVariables: function(
controller
)
{
var gl = controller.gl;
var program = controller.program;
var glDemo = controller.glDemo;
 
gl.validateProgram
(
program
);     

if (!gl.getProgramParameter
(
 program,
 gl.VALIDATE_STATUS
)
)
{   
   
 var validateError = gl.getProgramInfoLog
 (
  program
 );
   
 controller.viewGetWebGL
 (
  "Error while compiling the program:" + validateError
 ); 
   
 gl.deleteProgram(
  program
 );

 return;
}    
   
if (glDemo.init != null){
 glDemo.init(
  controller
 );
}

glDemo.render(
 controller
);  
},
/**
 * Obtain a reference
 * to WebGLContext.
 * @return {WebGLContext} or null.
 */
getGLContext: function(){
var canvas = this.cv;

var glContext = null;

try {
 
for (var i = 0; i < this.a3D.length; i++) {    
    
glContext = canvas.getContext(this.a3D[i]);
    
if (glContext != null && glContext != undefined && (typeof glContext !== 'undefined')) {  
break;
}
}    
} 
 
catch(err) {    
 this.viewGetWebGL(err.toString());       
}   
  
return glContext;
},


/**
 * Obtain a reference to
 * WebGLContext
 * with attributes.
 * @param {{attribute:value,...,atribute:value}} 
 * atribute.
 * @return {WebGLContext} or null.
 */
getGLContextAndAttribs: function(attributes){
var canvas = this.cv; 
var glContext = null;
try {
  
for (var i = 0; i < this.a3D.length; i++) {    
   
 glContext = canvas.getContext(
 	this.a3D[i],
  attributes
 );
    
 if (glContext != null) {
  break;
 }
}    
} 

catch(err) {    
 this.viewGetWebGL(err.toString());      
} 
 
return glContext;

},

/**
 * Prepare shaders.
 * @param {String} sID 
 * id to shader script
 * on the Web page.
 * @param {Number} nType  
 * WebGL constant
 * either:
 * FRAGMENT_SHADER or 
 * VERTEX_SHADER.
 */
getShader: function(
 sID, 
 nType
) {  
var gl = this.gl;
var sCode = "";   
var eShader = null;   
var nodeText; 
var shader = null;   
 
eShader = document.getElementById(
 sID
);   
 
nodeText = eShader.firstChild;
    
while(nodeText != null) {
 if (nodeText.nodeType == nodeText.TEXT_NODE) {         
  sCode += nodeText.textContent;
}     
     
nodeText = nodeText.nextSibling;
}
    
shader = gl.createShader(
 nType
);    

gl.shaderSource
(
 shader,
 sCode
);

gl.compileShader(
 shader
);         

if (!gl.getShaderParameter
(
 shader, 
 gl.COMPILE_STATUS
)) 
{  
    
 var sError = gl.getShaderInfoLog(
  shader
 );
   
 this.viewGetWebGL
 (      
  "An error occurred compiling the shaders: " + sError
 ); 

 return null;  
} 
 
return shader;

},

/**
 * Displays error messages
 * with optional non WebGL 
 * image link.
 * @param {String} s 
 * error message.
 */
viewGetWebGL: function(s){

if(s != null){
if (this.eDebug  == null){
this.eDebug = documentGetElementById('eDebug');
}
this.eDebug.innerHTML = s;
}

var cv = document.getElementById('cv');
cv.width = 0;
cv.height = 0;
		
var image = document.createElement("IMG");
image.src = "https://7Thunders.biz/assets/ssfb-7-all.jpg";
image.onclick=function(){window.location='https://7Thunders.biz/index.php';};
image.className="w100";	

var eGraphic = document.getElementById('eGraphic');
eGraphic.className="w100";
eGraphic.innerHTML = "<p>Your browser might not support WebGL.</p>";
eGraphic.innerHTML += "<p>For more information see <a href='http://get.webgl.org' title='Determine browser support of WebGL'>http://get.webgl.org</a>.</p>";
eGraphic.innerHTML += "<p>Tap the Graphic:</p>"
	eGraphic.innerHTML += "<p><a href='https://7Thunders.biz/index.php' title='Educational Games'>Educational Games</a></p>";

eGraphic.appendChild(image);
},

};
